"""
These are the available settings.

All attributes prefixed ``ADMIN_HELPERS_*`` can be overridden from your Django
project's settings module by defining a setting with the same name.
"""

from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from django.conf import settings as django_settings

# All attributes accessed with this prefix are possible to overwrite
# through django.conf.settings.
SETTINGS_PREFIX = "ADMIN_HELPERS_"


@dataclass(frozen=True)
class AppSettings:
    """Access this instance as `.conf.app_settings`."""

    ADMIN_HELPERS_URL_NAMESPACE: str = "admin"
    """URL namespace for the admin app (defaults to 'admin')."""

    def __getattribute__(self, __name: str) -> Any:
        """
        Check if a Django project settings should override the app default.

        In order to avoid returning any random properties of the django settings,
        we inspect the prefix firstly.
        """
        if __name.startswith(SETTINGS_PREFIX) and hasattr(django_settings, __name):
            return getattr(django_settings, __name)

        return super().__getattribute__(__name)


app_settings = AppSettings()



import pytest

from django_admin_helpers.links import (
    admin_add_url,
    admin_link_tag,
    admin_list_url,
    admin_url,
    inline_button,
    link_tag,
)

from .testapp.models import Author, Blog, Post


@pytest.mark.parametrize(
    ("model_class", "expected_url"),
    [
        (Blog, "/admin/testapp/blog/"),
        (Post, "/admin/testapp/post/"),
        (Author, "/admin/testapp/author/"),
    ],
)
def test_admin_list_url(model_class, expected_url):
    model_url = admin_list_url(model_class)
    instance = model_class()
    instance_url = admin_list_url(instance)
    assert instance_url == model_url == expected_url


def test_admin_list_url_with_setting(settings):
    settings.ADMIN_HELPERS_URL_NAMESPACE = "admin"
    url = admin_list_url(Blog)
    assert url == "/admin/testapp/blog/"


@pytest.mark.parametrize(
    ("model_class", "expected_url"),
    [
        (Blog, "/admin/testapp/blog/add/"),
        (Post, "/admin/testapp/post/add/"),
        (Author, "/admin/testapp/author/add/"),
    ],
)
def test_admin_add_url(model_class, expected_url):
    url = admin_add_url(model_class)
    assert url == expected_url


@pytest.mark.parametrize(
    ("model_class", "expected_url"),
    [
        (Blog, "/admin/testapp/blog/1/change/"),
        (Post, "/admin/testapp/post/1/change/"),
        (Author, "/admin/testapp/author/1/change/"),
    ],
)
def test_admin_url(model_class, expected_url):
    instance = model_class(id=1)
    instance_url = admin_url(instance)
    assert instance_url == expected_url


@pytest.mark.parametrize(
    ("label", "expected_str"),
    [
        (
            None,
            (
                '<a href="/admin/testapp/blog/1/change/">'
                "/admin/testapp/blog/1/change/</a>"
            ),
        ),
        (
            "Blog link",
            '<a href="/admin/testapp/blog/1/change/">Blog link</a>',
        ),
    ],
)
def test_admin_link_tag(label, expected_str):
    actual_str = admin_link_tag(Blog(id=1), label)
    assert str(actual_str) == expected_str


def test_admin_link_tag_no_instance():
    actual_str = admin_link_tag(None, "example")
    assert str(actual_str) == ""


@pytest.mark.parametrize(
    ("url", "label", "blank", "expected_str"),
    [
        (
            "/something/",
            "Link",
            False,
            '<a href="/something/">Link</a>',
        ),
        (
            "https://example.com",
            "External link",
            True,
            '<a href="https://example.com" target="_blank">External link</a>',
        ),
    ],
)
def test_link_tag(url, label, blank, expected_str):
    actual_str = link_tag(url, label, blank)
    assert str(actual_str) == expected_str


@pytest.mark.parametrize(
    ("extra_kwargs", "expected_styles"),
    [
        ({}, "margin-top: 0; margin-bottom: 1rem"),
        ({"mt": "0", "mb": "2rem"}, "margin-top: 0; margin-bottom: 2rem"),
        ({"mt": "1rem", "mb": "0"}, "margin-top: 1rem; margin-bottom: 0"),
    ],
)
def test_inline_button(extra_kwargs, expected_styles):
    actual_str = inline_button("/something/", "Link", **extra_kwargs)
    assert str(actual_str) == (
        "\n"
        '<a href="/something/" class="button"\n'
        f'   style="{expected_styles}; display: inline-block;">\n'
        "    Link\n"
        "</a>\n"
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from django.db.models import Model
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import SafeString, mark_safe

from .conf import app_settings

if TYPE_CHECKING:
    from django.utils.functional import Promise
    from typing_extensions import TypeAlias

    StrOrPromise: TypeAlias = str | Promise


def get_app_model_names(model: Model | type[Model]) -> tuple[str, str]:
    """Get app label and model name from private meta attribute."""
    return model._meta.app_label, model._meta.model_name


def admin_list_url(model: type[Model] | Model) -> str:
    """Get the URL to the admin change list for a model."""
    app_label, model_name = get_app_model_names(model)
    return reverse(
        f"{app_settings.ADMIN_HELPERS_URL_NAMESPACE}:"
        f"{app_label}_{model_name}_changelist"
    )


def admin_add_url(model: type[Model]) -> str:
    """Get the URL to the admin add for a model."""
    app_label, model_name = get_app_model_names(model)
    return reverse(
        f"{app_settings.ADMIN_HELPERS_URL_NAMESPACE}:{app_label}_{model_name}_add"
    )


def admin_url(instance: Model) -> str:
    """Get the URL to a model instance in the admin."""
    app_label, model_name = get_app_model_names(instance)
    return reverse(
        f"{app_settings.ADMIN_HELPERS_URL_NAMESPACE}:{app_label}_{model_name}_change",
        kwargs={"object_id": instance.pk},
    )


def admin_link_tag(
    instance: Model | None,
    label: StrOrPromise | None = None,
) -> SafeString:
    """Make a link to a model instance in the admin."""
    if not instance:
        return mark_safe("")
    label = label or str(instance)
    return link_tag(url=admin_url(instance), label=label)


def link_tag(
    url: str,
    label: StrOrPromise | None = None,
    blank: bool = False,
) -> SafeString:
    """Build an HTML tag for a hyperlink."""
    label = label or url
    tag_str = (
        '<a href="{url}" target="_blank">{label}</a>'
        if blank
        else '<a href="{url}">{label}</a>'
    )
    return format_html(tag_str, url=url, label=label)


INLINE_BUTTON_HTML = """
<a href="{url}" class="button"
   style="margin-top: {mt}; margin-bottom: {mb}; display: inline-block;">
    {label}
</a>
"""


def inline_button(
    url: str,
    label: StrOrPromise,
    mt: str = "0",
    mb: str = "1rem",
) -> str:
    """Build a button to be inlined in the admin."""
    return format_html(INLINE_BUTTON_HTML, url=url, label=label, mt=mt, mb=mb)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class AdminHelpersAppConfig(AppConfig):
    """App config for Django Admin Helpers."""

    name = "django_admin_helpers"
    verbose_name = _("admin helpers")
